@@ -3,7 +3,7 @@
 # Create a distributable archive of the current version of Makeself
 
 VER=`cat VERSION`
-mkdir /tmp/makeself-$VER
+mkdir -p /tmp/makeself-$VER release
 cp -a makeself* test README.md COPYING VERSION .gitmodules /tmp/makeself-$VER/
-./makeself.sh --notemp /tmp/makeself-$VER makeself-$VER.run "Makeself v$VER" echo "Makeself has extracted itself" 
+./makeself.sh --notemp /tmp/makeself-$VER release/makeself-$VER.run "Makeself v$VER" echo "Makeself has extracted itself" 
 
